searchData={"items":[{"type":"module","title":"LiveViewNative","doc":"This module should be used for creating new LiveView Native Clients.","ref":"LiveViewNative.html"},{"type":"module","title":"Configuration - LiveViewNative","doc":"To use LiveView Native you must configure application\nto register LiveView Native plugins and instruct\nPhoenix on how to handle certain formats.\n\n    # registers each plugin for use\n    config :live_view_native, plugins: [\n      LiveViewNative.SwiftUI,\n      LiveViewNative.Jetpack\n    ]\n\n    # configure the mimetype for each plugin's format\n    # refer to the plugin's documentation for each format\n    # in nearly all cases the content-type should be `\"text/:format\"`\n    config :mime, :types, %{\n      \"text/swiftui\" => [:swiftui],\n      \"text/jetpack\" => [:jetpack]\n    }\n\n    # configures the stylesheet for each format\n    # refer to the documentation of `LiveViewNative.Stylesheet`\n    # for more information on configuration options\n    config :live_view_native_stylesheet,\n      content: [\n        swiftui: [\n          \"lib/**/*swiftui*\"\n        ],\n        jetpack: [\n          \"lib/**/*jetpack*\"\n        ]\n      ]\n\n    # instructs Phoenix on how to encode a given format\n    config :phoenix_template, :format_encoders, [\n      swiftui: Phoenix.HTML.Engine,\n      jetpack: Phoenix.HTML.Engine\n    ]\n\n    # instructs Phoenix on which engine to\n    # use when compiling `neex` templates\n    config :phoenix, :template_engines,\n      neex: LiveViewNative.Engine\n\nNext you should add `LiveViewNative.LiveReloader` to your application's endpoint.\n\n    if code_reloading? do\n      socket \"/phoenix/live_reload/socket\", Phoenix.LiveReloader.Socket\n      plug Phoenix.LiveReloader\n\n      # add here to ensure it is only enabled\n      # when your application is configured for code reloading\n      plug LiveViewNative.LiveReloader\n\nFinally you will need to configure your router's plugs\n\n    plug :accepts, [\"html\", \"swiftui\", \"jetpack\"]\n    plug :put_root_layout,\n      html: {MyAppWeb.Layouts, :root},\n      swiftui: {MyAppWeb.Layouts.SwiftUI, :root},\n      jetpack: {MyAppWeb.Layouts.Jetpack, :root}\n\nRefer to the documentation in `LiveViewNative.Component` for more\ninformation on native layouts.\n\nYou are now ready to create your first LiveView Native app. Refer to\nthe documentation in `LiveViewNative.LiveView` on how to configure\nyour live views for native.","ref":"LiveViewNative.html#module-configuration"},{"type":"module","title":"Enabling LiveView for Native - LiveViewNative","doc":"Now that your application is configured you will need to enable a LiveView for Native.\n\nAssuming your application has the following LiveView route:\n\n    live \"/\", HomeLive\n\nYou'll need to add the `user MyAppNative, :live_view` to that module:\n\n    defmodule MyAppWeb.HomeLive do\n      use MyAppWeb, :live_view\n      use MyAppNative, :live_view\n\nNext you should use the `mix lvn.gen.live` task to generate\nthe LiveView Native render component and template.\n\n    > mix lvn.gen.live swiftui Home\n\nThis will generate the following files in your project:\n\n  * lib/my_app_web/live/home_live.swiftui.ex\n  * lib/my_app_web/live/swiftui/home_live.swiftui.neex\n\nJust like a regular LiveView you can decide not to use the template and render\ntemplates in-line inside the render component:\n\n    defmodule MyAppWeb.HomeLive.SwiftUI do\n      use MyAppNative, [:render_component, format: :swiftui]\n\n      def render(assigns, _interface) do\n        ~LVN\"\"\"\n         Hello, LiveView Native! \n        \"\"\"\n      end\n    end\n\nNote that unlike a regular LiveView that has `render/1` the\nLiveView Native render component uses `render/2` as the 2nd argument\nwill include interface information about the client.\n\nYou can read more in `LiveViewNative.Component`","ref":"LiveViewNative.html#module-enabling-liveview-for-native"},{"type":"macro","title":"LiveViewNative.__using__/1","doc":"Uses LiveViewNative for creating new clients\n\n    defmodule GameBoy do\n      use LiveViewNative,\n        format: :gameboy,\n        component: GameBoy.Component,\n        module_suffix: GameBoy,\n        template_engine: LiveViewNative.Engine\n    end","ref":"LiveViewNative.html#__using__/1"},{"type":"macro","title":"Options - LiveViewNative.__using__/1","doc":"* `:format` - the format that will be used by Phoenix to determine the request\n  and response encoding. Should be all lowercase and underscored\n  * `:component` - module name of the client's component module to be used within\n  render components\n  * `:module_suffix` - the module name that will be assumed to be appended to\n  liveviews for render components (i.e. MyAppWeb.HomeLive.GameBoy)\n  * `:template_engine` - the engine for compiling a client's template. In nearly\n  all cases `LiveViewNative.Engine` will be sufficient","ref":"LiveViewNative.html#__using__/1-options"},{"type":"function","title":"LiveViewNative.available_formats/0","doc":"Returns a list of all available formats\n\nThe format list is derived from the plugins returned by `LiveViewNative.plugins/0`","ref":"LiveViewNative.html#available_formats/0"},{"type":"function","title":"LiveViewNative.fetch_plugin/1","doc":"Fetches a plugin based upon the format name\n\nFollows the same return types as `Map.fetch/2`","ref":"LiveViewNative.html#fetch_plugin/1"},{"type":"function","title":"LiveViewNative.fetch_plugin!/1","doc":"Fetches a plugin based upon the format name\n\nIf the format is not present `LiveViewNative.PluginError` is raised","ref":"LiveViewNative.html#fetch_plugin!/1"},{"type":"function","title":"LiveViewNative.plugins/0","doc":"Returns a list of all available LiveView Native plugins\n\nOnly the plugins that have been registered in your application\nconfig will be returned in the list","ref":"LiveViewNative.html#plugins/0"},{"type":"module","title":"LiveViewNative.Component","doc":"Define reusable LiveView Native function components with NEEx templates.\n\nThis module is used for the following\n\n* format specific render components\n* format specific functional components\n* format specific layouts\n\nLiveView Native Components differ from Phoenix Components in that they make\nuse of a two-argument function instead of a single argument function. For example,\nwith a Phoenix Component that has a template named `foo_bar.html.heex` it would define\n`foo_bar/1` as the rendering function for that template when embeded. LiveView Native Components\nwould definee `foo_bar/2` for `foo_bar.swiftui.neex` with the 1st argument still being `assigns` and\nthe 2nd argument being `interface` map with information about the native client that is connecting.\n\nFor more information on template embedding see `LiveViewNative.Renderer.embed_templates/2`","ref":"LiveViewNative.Component.html"},{"type":"module","title":"Interface map - LiveViewNative.Component","doc":"Each native client can optionally send along an interface map that has at least the following information:\n\n  * `\"target\"` - the target device within the platform family. For example, `ios`, `ipados`, `watchos` are all targets\n  for the SwiftUI platform. Each client library should have information on the valid targets\n  * `\"version\"` - the version of your application. This is a value that you can control and pattern match on. The `version`\n  will need to be set in the native client build.\n  * `\"client_version\"` - the version of the client library that is connecting. The client will set this.","ref":"LiveViewNative.Component.html#module-interface-map"},{"type":"macro","title":"LiveViewNative.Component.__using__/1","doc":"Uses LiveView Native Component in the current module","ref":"LiveViewNative.Component.html#__using__/1"},{"type":"macro","title":"Options - LiveViewNative.Component.__using__/1","doc":"* `:format` - the format that corresponds to a LiveView Native client format for this component\n  * `:root` - the directory relative to the component module's location where templates are stored.\n  See `LiveViewNative.Renderer.embed_templates/2` for more information.\n  * `:as` - will override the name of the rendering function injected into the module.\n  See `LiveViewNative.Renderer.embed_templates/2` for more information.","ref":"LiveViewNative.Component.html#__using__/1-options"},{"type":"macro","title":"Format-Specific Render Component - LiveViewNative.Component.__using__/1","doc":"defmodule MyAppWeb.HomeLive.SwiftUI do\n      use LiveViewNative.Component,\n        format: :swiftui,\n        as: :render\n    end\n\nIn this example `:as` is used and this will inject following:\n\n    embed_templates(\"swiftui/home_live*\", root: nil, name: :render)\n\nAssuming there is a `swiftui/home_live.swiftui.neex` this will be embeded in the render\ncomponent as `render/2`.\n\nAlternatively if you do not want to use templates but the in-line rendering you can declare your own\n`render/2` functions\n\n    def render(assigns, %{\"target\" => \"watchos\"}) do\n      ~LVN\"\"\"\n       Hello, from WatchOS! \n      \"\"\"\n    end\n\n    def render(assigns, %{\"target\" => target}) when target in ~w{macos tvos} do\n      ~LVN\"\"\"\n       Hello, from <%= target %>! \n      \"\"\"\n    end\n\n    def render(assigns, _interface) do\n      ~LVN\"\"\"\n       Hello, from SwiftUI! \n      \"\"\"\n    end","ref":"LiveViewNative.Component.html#__using__/1-format-specific-render-component"},{"type":"macro","title":"Format-Specific Function Component - LiveViewNative.Component.__using__/1","doc":"defmodule MyAppWeb.NativeComponent.SwiftUI do\n      use LiveViewNative.Component,\n        format: :swiftui\n    end\n\nWhen using to write your own function components it is nearly identical to `Phoenix.Component`.\nYou will use `embed_templates/2` for template embedding and this will inject functions that take\n`assigns` and `interface` as arguments.\n\nYou can also define your own functions but you lose the 2 arity, or you can create it yourself.\n\n    def radio_button(assigns) do\n      interface = LiveViewNative.Utils.get_interface(assigns)\n      radio_button(assign, interface)\n    end\n\n    def radio_button(assigns, %{\"target\" => \"watchos\"}) do\n      ~LVN\"\"\"\n        ...\n      \"\"\"\n    end","ref":"LiveViewNative.Component.html#__using__/1-format-specific-function-component"},{"type":"macro","title":"Format-Specific Layouts - LiveViewNative.Component.__using__/1","doc":"defmodule MyAppWeb.Layouts.SwiftUI do\n      use LiveViewNative.Component,\n        format: :swiftui\n\n      import LiveViewNative.Component, only: [csrf_token: 1]\n\n      embed_templates \"layouts_swiftui/*\"\n    end\n\nMost times you will want to provide `app` and `root` templates.\nFor stylesheet embedding refer to `LiveViewNative.Stylesheet.Component`","ref":"LiveViewNative.Component.html#__using__/1-format-specific-layouts"},{"type":"function","title":"LiveViewNative.Component.csrf_token/1","doc":"Embed the CSRF token for LiveView as a tag","ref":"LiveViewNative.Component.html#csrf_token/1"},{"type":"macro","title":"LiveViewNative.Component.sigil_LVN/2","doc":"The `~LVN` sigil for writing HEEx templates inside source files.\n\nUsing `~LVN` is nearly identical to `~H` in `Phoenix.Component` but deviates\nin various ways.\n\nOne common misconception is that LiveView Native templates are just HTML or XML. This is not true.\nBoth HTML and XML have specifications that at times conflict with the needs of how LiveView Native client\nUI frameworks should be represented. Think of LiveView Native templates as a composable markup whose specification\nis currently under development. As we continue to expand the LiveView Native ecosystem this list will likely grow:\n\n* casing - `~LVN` does not enforce downcasing of tag names so ` ` is a valid tag name","ref":"LiveViewNative.Component.html#sigil_LVN/2"},{"type":"module","title":"LiveViewNative.ContentNegotiator","doc":"The callback module for delegating LiveView Native format\nrequests to the correct render component","ref":"LiveViewNative.ContentNegotiator.html"},{"type":"module","title":"LiveViewNative.Engine","doc":"The LiveViewNative.Engine that powers `.neex` templates and the `~LVN` sigil.\n\nIt works by adding a LiveView Native template parsing and validation layer on top\nof `Phoenix.LiveView.TagEngine`.","ref":"LiveViewNative.Engine.html"},{"type":"function","title":"LiveViewNative.Engine.encode_to_iodata!/1","doc":"Encodes the HTML templates to iodata.","ref":"LiveViewNative.Engine.html#encode_to_iodata!/1"},{"type":"module","title":"LiveViewNative.LiveReloader","doc":"Router for live-reload detection of LiveView Native applications in development.\n\n# Usage\n\nAdd the `LiveViewNative.LiveReloader` plug within a `code_reloading?` block\nin your Endpoint, ie:\n\n    if code_reloading? do\n      socket \"/phoenix/live_reload/socket\", Phoenix.LiveReloader.Socket\n      plug Phoenix.LiveReloader\n      plug LiveViewNative.LiveReloader\n    end","ref":"LiveViewNative.LiveReloader.html"},{"type":"function","title":"LiveViewNative.LiveReloader.call/2","doc":"","ref":"LiveViewNative.LiveReloader.html#call/2"},{"type":"function","title":"LiveViewNative.LiveReloader.init/1","doc":"","ref":"LiveViewNative.LiveReloader.html#init/1"},{"type":"module","title":"LiveViewNative.LiveView","doc":"Use this module within a LiveView to delegate rendering requests\nfrom a native client to a format specific rendering component.\n\nAll event handling should remain in the LiveView, all rendering\nconcerns for a given format will be delegated to the component.\n\nPlease refer to `LiveViewNative.Component` for more information\non rendering components.","ref":"LiveViewNative.LiveView.html"},{"type":"macro","title":"LiveViewNative.LiveView.__using__/1","doc":"Uses LiveView Native in the current module for rendering delegation\n\n    defmodule MyAppWeb.HomeLive do\n      use MyAppWeb, :live_view\n      use LiveViewNative,\n        formats: [:swiftui, :jetpack],\n        layouts: [\n          swiftui: {MyAppWeb.Layouts.SwiftUI, :app},\n          jetpack: {MyAppWeb.Layouts.Jetpack, :app}\n        ]\n    end","ref":"LiveViewNative.LiveView.html#__using__/1"},{"type":"macro","title":"Options - LiveViewNative.LiveView.__using__/1","doc":"* `:formats` - the formats this LiveView will delegate to\n    a render component. For example, specifying `formats: [:swiftui, jetpack]`\n    for a LiveView named `MyAppWeb.HomeLive` will\n    invoke `MyAppWeb.HomeLive.SwiftUI` and `MyAppWeb.HomeLivew.Jetpack` when\n    respectively rendering each format. The appended module suffix\n    is taken from the `:module_suffix` value on each registered LiveView Native\n    plugin.\n\n  * `:layouts` - which layouts to render for each format,\n    for example: `[swiftui: {MyAppWeb.Layouts.SwiftUI, :app}]`","ref":"LiveViewNative.LiveView.html#__using__/1-options"},{"type":"module","title":"LiveViewNative.Renderer","doc":"This module contains logic on how template rendering and embedding is implemented","ref":"LiveViewNative.Renderer.html"},{"type":"macro","title":"LiveViewNative.Renderer.embed_templates/2","doc":"Embeds templates as render functions\n\nThis function is mostly identical to `Phoenix.Component.embed_templates/2` but deviates in a few ways","ref":"LiveViewNative.Renderer.html#embed_templates/2"},{"type":"macro","title":"Options - LiveViewNative.Renderer.embed_templates/2","doc":"* `:format` - the format from the client. This will be used for template discovery, for the format\n  `:swiftui` templates such as `home_live.swiftui.neex` will collected.\n  * `:name` - the name of the generated function. Defaults to the name of the template, for example\n  `app.swiftui.neex` will generate `app/2`. If this value is passed in there must only be a single template\n  name matched. So if the pattern results in `['home_live.swiftui.neex', 'user_list.swiftui.neex']` and\n  you pass `name: :render` there is ambiguity and will `raise`. The exception is when there are multiple\n  targets for the same name: `['home_live.swiftui+watchos.neex', 'home_live.swiftui+tvos.neex', 'home_live.swiftui.neex']`\n  if `name: :render` this will result:\n\n    def render(assings, %{\"target\" => \"watchos\"} = interface)\n    def render(assigns, %{\"target\" => \"tvos\"} = interface)\n    def render(assigns, interface)\n\n  the fallback function will always sort to render last. This allows for a greedy matcher if more than one target\n  will be used for the same template.\n\n  * `:root` - the location from which the `pattern` is applied to for finding templates. This is relative\n  to the rendering module.","ref":"LiveViewNative.Renderer.html#embed_templates/2-options"},{"type":"module","title":"LiveViewNative.TagEngine","doc":"For more information on TagEngine please see `Phoenix.LiveView.TagEngine`","ref":"LiveViewNative.TagEngine.html"},{"type":"module","title":"LiveViewNative.Template","doc":"","ref":"LiveViewNative.Template.html"},{"type":"function","title":"LiveViewNative.Template.attributes_escape/1","doc":"","ref":"LiveViewNative.Template.html#attributes_escape/1"},{"type":"function","title":"LiveViewNative.Template.engines/0","doc":"Return all template engines for registered plugins","ref":"LiveViewNative.Template.html#engines/0"},{"type":"function","title":"LiveViewNative.Template.list_value/2","doc":"","ref":"LiveViewNative.Template.html#list_value/2"},{"type":"function","title":"LiveViewNative.Template.style_value/1","doc":"","ref":"LiveViewNative.Template.html#style_value/1"},{"type":"protocol","title":"LiveViewNative.Template.Safe","doc":"Defines the LVN Template safe protocol.\n\nThis protocol relies on iodata, which provides\nbetter performance when sending or streaming data to the client.","ref":"LiveViewNative.Template.Safe.html"},{"type":"function","title":"LiveViewNative.Template.Safe.to_iodata/1","doc":"","ref":"LiveViewNative.Template.Safe.html#to_iodata/1"},{"type":"type","title":"LiveViewNative.Template.Safe.t/0","doc":"All the types that implement this protocol.","ref":"LiveViewNative.Template.Safe.html#t:t/0"},{"type":"module","title":"Mix.LiveViewNative","doc":"","ref":"Mix.LiveViewNative.html"},{"type":"function","title":"Mix.LiveViewNative.plugins/0","doc":"","ref":"Mix.LiveViewNative.html#plugins/0"},{"type":"task","title":"mix lvn","doc":"Prints LiveView Native tasks and their information.\n\n    $ mix lvn\n\nTo print the LiveView Native version, pass `-v` or `--version`, for example:\n\n    $ mix lvn --version","ref":"Mix.Tasks.Lvn.html"},{"type":"task","title":"mix lvn.gen","doc":"Generates the Native module\n\n    $ mix lvn.gen\n\nInstructions will be printed for configuring your application. And a\nnew `Native` module will be copied into the `lib/` directory of your application.","ref":"Mix.Tasks.Lvn.Gen.html"},{"type":"task","title":"Options - mix lvn.gen","doc":"* `--no-live-form` - don't include `LiveViewNative.LiveForm` content in the `Native` module\n* `--no-gettext` - don't include Gettext support","ref":"Mix.Tasks.Lvn.Gen.html#module-options"},{"type":"function","title":"Mix.Tasks.Lvn.Gen.gen/1","doc":"","ref":"Mix.Tasks.Lvn.Gen.html#gen/1"},{"type":"task","title":"mix lvn.gen.layout","doc":"Generates a new format specific layout component and templates\n\n    $ mix lvn.gen.layout swiftui","ref":"Mix.Tasks.Lvn.Gen.Layout.html"},{"type":"task","title":"mix lvn.gen.live","doc":"Generates a new format specific LiveView render component and template\n\n    $ mix lvn.gen.live swiftui Home","ref":"Mix.Tasks.Lvn.Gen.Live.html"},{"type":"task","title":"mix lvn.setup","doc":"@{shortdoc}\n\n    $ mix lvn.setup","ref":"Mix.Tasks.Lvn.Setup.html"},{"type":"task","title":"mix lvn.setup.config","doc":"Configure LiveView Native within a Phoenix LiveView application\n\nThis setup will\n\n    $ mix lvn.setup.config","ref":"Mix.Tasks.Lvn.Setup.Config.html"},{"type":"task","title":"mix lvn.setup.gen","doc":"Run LiveView Native setup generators within a Phoenix LiveView application\n\nThis setup will\n\n    $ mix lvn.setup.config","ref":"Mix.Tasks.Lvn.Setup.Gen.html"},{"type":"task","title":"Options - mix lvn.setup.gen","doc":"* `--no-gettext` - don't include Gettext support","ref":"Mix.Tasks.Lvn.Setup.Gen.html#module-options"},{"type":"function","title":"Mix.Tasks.Lvn.Setup.Gen.generators/1","doc":"","ref":"Mix.Tasks.Lvn.Setup.Gen.html#generators/1"},{"type":"exception","title":"LiveViewNative.PluginError","doc":"","ref":"LiveViewNative.PluginError.html"},{"type":"extras","title":"LiveViewNative","doc":"# LiveViewNative\n\n[![Build Status](https://github.com/liveview-native/live_view_native/workflows/Elixir%20CI/badge.svg)](https://github.com/liveview-native/live_view_native/actions) [![Hex.pm](https://img.shields.io/hexpm/v/live_view_native.svg)](https://hex.pm/packages/live_view_native) [![Documentation](https://img.shields.io/badge/documentation-gray)](https://hexdocs.pm/live_view_native)","ref":"readme.html"},{"type":"extras","title":"About - LiveViewNative","doc":"LiveView Native is a platform for building native applications using [Elixir](https://elixir-lang.org/) and [Phoenix LiveView](https://github.com/phoenixframework/phoenix_live_view). It allows a single LiveView to serve both web and non-web clients by transforming platform-specific template code into native UIs:\n\n```elixir\n# lib/my_app_web/live/hello_live.ex\ndefmodule MyAppWeb.HelloLive do\n  use MyAppWeb, :live_view\n  use MyAppNative, :live_view\nend\n\n# liv/my_app_web/live/hello_live_swiftui.ex\ndefmodule MyAppWeb.HelloLive.SwiftUI do\n  use MyAppNative, [:render_component, format: :swiftui]\n\n  def render(assigns, %{\"target\" => \"watchos\"}) do\n    ~LVN\"\"\"\n     \n       \n        Hello WatchOS!\n       \n     \n    \"\"\"\n  end\n\n  def render(assigns, _interface) do\n    ~LVN\"\"\"\n     \n       \n        Hello SwiftUI!\n       \n     \n    \"\"\"\n  end\nend\n```","ref":"readme.html#about"},{"type":"extras","title":"Getting started - LiveViewNative","doc":"To get started with LiveView Native, you'll need to have an existing [Phoenix Application](https://hexdocs.pm/phoenix/up_and_running.html) or create a new one.\n\nAdd `live_view_native` to your list of dependencies in the `mix.exs` file. In addition to `live_view_native` you may want to include some additional libraries:\n\n```elixir\n{:live_view_native, \"~> 0.3.0\"},\n{:live_view_native_stylesheet, \"~> 0.3.0\"},\n{:live_view_native_swiftui, \"~> 0.3.0\"},\n{:live_view_native_live_form, \"~> 0.3.0\"}\n```\n\nThen run:\n\n```\n$ mix lvn.setup\n```\n\nand follow the instructions on how to complete the setup process.","ref":"readme.html#getting-started"},{"type":"extras","title":"Native Clients - LiveViewNative","doc":"LiveView Native enables client frameworks such as:\n\n| UI Framework     | Devices                                              | LiveView Client |\n|------------------|------------------------------------------------------|-----------------|\n| SwiftUI          | iPhone, iPad, AppleTV, Apple Watch, MacOS, Apple Vision Pro | [LiveView Native SwiftUI](https://github.com/liveview-native/liveview-client-swiftui) |\n| JetPack Compose  | Android family                                       | [LiveView Native Jetpack](https://github.com/liveview-native/liveview-client-jetpack) |\n| HTML             |                                                    | [LiveView Native HTML](https://github.com/liveview-native/liveview-client-html) |","ref":"readme.html#native-clients"},{"type":"extras","title":"Questions? - LiveViewNative","doc":"Have a question or want some help with LiveView Native?\n\nCheck out the `#liveview-native` channel on the [Elixir Lang Slack](https://elixir-lang.slack.com/).","ref":"readme.html#questions"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}